/*
 Copyright (C) 2007 by Ben Anderman <ben@happyspork.com>
 Part of Sporktris http://www.happyspork.com/games
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License.
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY.
 
 See the License.txt file for more details.
 */

#include "common_funcs.h"
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <string>
#include <stdlib.h>

using namespace std;

SDL_Surface *funcs::load_surf (string img_path, int alpha, const char *name)
{
	SDL_Surface *temp = IMG_Load(img_path.c_str());
	SDL_Surface *surf;
	die_surf(temp, name);
	if (alpha)
	{
		surf = SDL_DisplayFormatAlpha(temp);
	} else {
		surf = SDL_DisplayFormat(temp);
	}
	die_surf(surf, name);
	if (alpha > 1)
	{
		SDL_SetAlpha(surf, SDL_SRCALPHA | SDL_RLEACCEL, 0);
	} else {
		SDL_SetAlpha(surf, 0, 0);
	}
	SDL_FreeSurface(temp);
	return(surf);
}

SDL_Surface *funcs::load_surf (int width, int height, const char *name)
{
	SDL_Surface *temp = create_compatible_surface(SDL_GetVideoSurface(), width, height);
	die_surf(temp, name);
	SDL_Surface *surf = SDL_DisplayFormatAlpha(temp);
	die_surf(surf, name);
	SDL_SetAlpha(surf, SDL_SRCALPHA, 0);
	return(surf);
}

SDL_Surface *funcs::create_compatible_surface(SDL_Surface* surf, int width, int height)
{
	//copied from Wesnoth - thanks Sirp :)
	if(width == -1)
		width = surf->w;
	if(height == -1)
		height = surf->h;
	return SDL_CreateRGBSurface(SDL_SWSURFACE,width,height,surf->format->BitsPerPixel,surf->format->Rmask,surf->format->Gmask,surf->format->Bmask,surf->format->Amask);
}

void funcs::die_surf (SDL_Surface *surf, const char *name)
{
	if (surf == NULL)
	{
		printf("couldn't load \"%s\", gotta go!\n", name);
		exit(1);
	}
}

string funcs::str_int (int n)
{
	char chr[sizeof(int)*8+1];
	sprintf(chr, "%i", n);
	return(string(chr));
}

void funcs::init_ttf (void)
{
	if (!TTF_WasInit())
	{
		if (TTF_Init() == -1)
		{
			printf("Oh no! we can't render text! i asked, but all they said was: %s - those bastards!\n", TTF_GetError());
			exit(1);
		} else {
			atexit(TTF_Quit);
		}
	}
}

SDL_Color funcs::make_color (string str)
{
	if (str.length() >= 6)
	{
		Uint8 color[3];
		for (int i = 0; i < 6; i+=2)
		{
			color[i/2] = (char)strtol(str.substr(i, 2).c_str(), NULL, 16);
		}
		SDL_Color result = {color[0], color[1], color[2], 0};
		return (result);
	} else {
		SDL_Color result = {0, 0, 0, 0};
		return (result);
	}
}

string funcs::expand_path (string path)
{
	if (path.length() > 0 && path[0] == '~')
	{
		string home(getenv("HOME"));
		path = home + path.substr(1);
	}
	return path;
}
